#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include <fcall.h>
#include <mp.h>
#include <libsec.h>
#include <thread.h>
#include <pool.h>
#include <ip.h>
#include <9p.h>
#include "nbcache.h"

int Debug;
int Vendor;
int Probe;
int Pid;

void
fsopen(Req *r)
{
	r->fid->file->aux = snapshot();
	respond(r, nil);
}

static void
fsread(Req *r)
{
	readstr(r, r->fid->file->aux);
	respond(r, nil);
}

void
fsdestroyfile(File *f)
{
	free(f->aux);
}


static void
fsend(Srv *srv)
{
	USED(srv);

	postnote(PNPROC, Pid, "die");
}

Srv fs = {
	.open = fsopen,
	.read = fsread,
	.end= fsend,
};

static void
usage(void)
{
	fprint(2, "usage: %s [-p] [-m mntpt] [-s srvname] [-f filename] [-x /net]\n", argv0);
	exits("usage");
}

static void
ding(void *u, char *msg)
{
	USED(u);
	if(strstr(msg, "alarm"))
		noted(NCONT);
	noted(NDFLT);
}


void
main(int argc, char *argv[])
{
	File *f;
	char *mtpt, *svs, *file;

	svs = nil;
	Netdir = "/net";
	mtpt = "/lib/ndb";
	file = "gleaned";
	ARGBEGIN{
	case 'd':
		Debug++;
		break;
	case 'p':
		Probe = 1;
		break;
	case 'D':
		chatty9p++;
		break;
	case 'm':
		mtpt = EARGF(usage());
		break;
	case 'f':
		file = EARGF(usage());
		break;
	case 's':
		svs = EARGF(usage());
		break;
	case 'v':
		Vendor = 1;
		break;
	case 'x':
		Netdir = EARGF(usage());
		break;
	case '?':
		usage();
		break;
	default:
		fprint(2, "unrecognized option\n");
		usage();
	}ARGEND

	if (argc != 0)
		usage();

	notify(ding);

	quotefmtinstall();
	fmtinstall('I', eipfmt);
	fmtinstall('V', eipfmt);
	fmtinstall('E', eipfmt);
	fmtinstall('M', eipfmt);

	Pid = rfork(RFPROC|RFNOWAIT|RFMEM|RFNOTEG|RFNAMEG);
	switch(Pid){
	case -1:
		sysfatal("fork() failed %r\n");
	case 0:
		listener();
		break;
	default:
		break;
	}

	fs.tree = alloctree(nil, nil, DMDIR|0755, fsdestroyfile);
	postmountsrv(&fs, svs, mtpt, MBEFORE);
	if((f = createfile(fs.tree->root, file, nil, 0444, nil)) == nil)
		sysfatal("cannot create file %r");
	closefile(f);

	exits(0);
}

